
;(function(exports, undefined) {

  var EventVat = exports.EventVat = function EventVat(conf) {

    if (!(this instanceof EventVat)) {
      return new EventVat(conf);
    }

    var self = this;
    
    if (!self._listeners) self.initializeEvents(conf);
    
    self.store = conf && conf.data || {};
    
    self._deathRate = 10;
    self.keys = [];
    self.killer = setInterval(function() {
    
      var key = self.keys.shift();
      var keyExists = !!self.store[key];
    
      if(keyExists && self.store[key].dueDate === Date.now()) {
        self.del(key);
        return true;
      }

      if(keyExists && self.store[key].ttl && self.store[key].ttl < 0) {
        self.del(key);
      }
      else if(keyExists && self.store[key].ttl) {
        (self.store[key].ttl--);
        keys.push(key);
      }

      return true;
      
    }, self._deathRate);

    return this;
  };

  if(!EventEmitter2) {
    throw new Error('EventEmitter2 is not defined.');
  }
  
  for (var member in EventEmitter2.prototype) {
    EventVat.prototype[member] = EventEmitter2.prototype[member];
  }
  
  EventVat.prototype.get = function(key) {

    var newValue;

    if(this.store[key]) {
      var newValue = this.store[key].value;
      this.emit('get.' + key, newValue);
      this.emit('get', newValue);
      return newValue;
    }
    else {
      return false;
    }
  };

  EventVat.prototype.set = function(key, value, ttl) {
    var self = this;
    
    if(this.store[key]) {
      this.store[key].ttl = ttl;
      this.store[key].value = value;
    }
    else {
      this.store[key] = {
        value: value,
        ttl: ttl // time to live
      };

      this.keys.push(key);
    }

    this.emit('set.' + key, value);
    this.emit('set', key, value);
    return value;
  };
  
  EventVat.prototype.setnx = function(key, value) {
    if(!this.store[key]) {
      this.store[key] = value;
      this.emit('set.' + key, value);
      this.emit('set');
      return this.store[key].value;
    }
    else {
      return false;
    }
  };
  
  EventVat.prototype.rename = function(oldKey, newKey) {
    if(this.store[oldKey]) {
      this.store[newKey] = this.store[oldKey];
      this.emit('rename.' + oldKey, newKey);
      this.emit('rename', oldKey, newKey);
      return this.store[key].value;
    }
    else {
      return false;
    }
  };
  
  EventVat.prototype.renamenx = function(oldKey, newKey) {
    if(this.store[oldKey] && typeof this.store[newKey] === 'undefined') {
      this.store[newKey] = this.store[oldKey];
      this.emit('renamenx.' + oldKey, newKey);
      this.emit('renamenx', oldKey, newKey);
      return this.store[key].value;
    }
    else {
      return false;
    }
  };  
  
  EventVat.prototype.decr = function(key, value) {
    if(this.store[key] && typeof this.store[key].value === 'number') {
      var newValue = value ? this.store[newKey].value -= value : this.store[newKey].value--;
      this.emit('decr.' + key, value, newValue);
      this.emit('decr', value, newValue);
      return newValue;
    }
    else {
      return false;
    }
  };
  
  EventVat.prototype.incr = function(key, value) {
    if(this.store[key] && typeof this.store[key].value === 'number') {
      value ? this.store[newKey].value += value : this.store[newKey].value++;
      this.emit('incr.' + key, value);
      this.emit('incr', value);
      return newValue;
    }
    else {
      return false;
    }
  };
  
  EventVat.prototype.swap = function(a, b, depth) {
    if(this.store[a] && this.store[b]) {
      
      a = this.store[a]; 
      b = this.store[b];
      
      if(depth) {
        a = [b, b = a][0];
      }
      else {
        a.value = [b.value, b.value = a.value][0];
      }

      this.emit('swap.' + a, b, depth);
      this.emit('swap', a, b, depth);
      return depth ? [a, b] : [a.value, b.value];
    }
    else {
      return false;
    }

  };

  EventVat.prototype.findin = function(key, value) {
    if(this.store[key]) {
      var index = (~this.store[key].value.indexOf(value));
      this.emit('findin.' + key, value, index);
      this.emit('findin', value, index);
      return index;
    }
    else {
      return false;
    }
  };

  EventVat.prototype.del = function(key) {
    if(!this.store[key]) {
      this.emit('del.' + key);
      this.emit('del');
      return delete this.store[key];
    }
    else {
      return false;
    }
  };

  EventVat.prototype.exists = function(key) {
    if(this.store[key]) {
      this.emit('exists.' + key);
      this.emit('exists');
      return true;
    }
    else {
      return false;
    }
  };

  EventVat.prototype.expires = function(key, ttl) {

    var self = this, newVal;
    var op = ttl[0];

    ttl = parseInt(ttl.slice(1), 10);

    if(op === '+') { 
      newVal = this.store[key].ttl += ttl;
    }
    else {
      newVal = this.store[key].ttl -= ttl;
    }

    return newVal;
  };  
  
  EventVat.prototype.expireat = function(key, dueDate) {
    if(this.store[key]) {
      return this.store[key].dueDate = dueDate;
    }
    else {
      return false;
    }
  };

  EventVat.prototype.keys = function(key) {

    if(this.store[key]) {
      var keys = [];

      for(var k in this.store) {
        if(regex.test(key)) {
          keys.push(k);
        }
      }

      return keys;
    }
    else {
      return false;
    }
  };

  EventVat.prototype.persist = function(key) {
    if(this.store[key] && self.store[key].ttl) {
      this.emit('persist.' + key);
      this.emit('persist');
      return delete this.store[key].ttl;
    }
    else {
      return false;
    }
  };
  
  EventVat.prototype.randomkey = function() {
    var keys = Object.keys(this.store);
    var index = Math.floor(Math.random()*keys.length);
    return keys[index];
  };
  
  EventVat.prototype.type = function(key) {
    return ({}).toString.call(this.store[key]);
  };
  
  EventVat.prototype.append = function(key, value) {
    if(this.store[key] && typeof this.store[key] === 'string') {
      var newValue = this.store[key].value + value;
      this.emit('append.' + key, value, newValue);
      this.emit('append', value, newValue);
      return newValue;
    }
    else {
      return false;
    }
  };  

  EventVat.prototype.ttl = function(key, ttl) {
    return this.store[key].ttl;
  };

  EventVat.prototype.dump = function(stringify) {
    
    var dump = {};
    
    for(var key in this.store) {
      if(this.store.hasOwnProperty(key)) {
        dump[key] = this.store[key].value;
      }
    }
    
    dump = stringify ? JSON.stringify(dump) : dump;
    
    this.emit('dump', null, dump);
    return dump;
  };

}((typeof exports === 'undefined') ? window : exports));

